/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.paimon.spark

import org.apache.spark.sql.connector.read.Statistics

import java.util.OptionalLong

case class PaimonStatistics[T <: PaimonBaseScan](scan: T) extends Statistics {

  private lazy val rowCount: Long = scan.getSplits.map(_.rowCount).sum

  private lazy val scannedTotalSize: Long = rowCount * scan.readSchema().defaultSize

  override def sizeInBytes(): OptionalLong = OptionalLong.of(scannedTotalSize)

  override def numRows(): OptionalLong = OptionalLong.of(rowCount)

  // TODO: extend columnStats for CBO
}
